﻿using AdventOfCode.Common;
using AdventOfCode.Models;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;

namespace AdventOfCode._2022
{
    [AOC(year: 2022, day: 2)]
    public class Day2 : AOCDay
    {
        protected override AOCResponse ExecutePartA()
        {
            var rows = GetSplitInput();
            var totalScore = 0;
            foreach (var row in rows)
            {
                var opponent = row.Split(" ")[0];
                var mine = row.Split(" ")[1];
                var score = ScoreRound(opponent, mine);
                Log(score);
                totalScore += score;
            }
            _response.Answer = totalScore;
            return _response;
        }

        protected override AOCResponse ExecutePartB()
        {
            var rows = GetSplitInput();
            var totalScore = 0;
            foreach (var row in rows)
            {
                var opponent = row.Split(" ")[0];
                var mine = DeduceCorrectShape(opponent, row.Split(" ")[1]);
                var score = ScoreRound(opponent, mine);
                Log(score);
                totalScore += score;
            }
            _response.Answer = totalScore;
            return _response;
        }

        private string DeduceCorrectShape(string opponent, string mine)
        {
            var oppScore = GetShapeScore(opponent);
            if (mine == "X") //Lose
            {
                if (oppScore == 1) return "C";
                else if (oppScore == 2) return "A";
                else if (oppScore == 3) return "B";
            }else if (mine == "Y") //Draw
            {
                if (oppScore == 1) return "A";
                else if (oppScore == 2) return "B";
                else if (oppScore == 3) return "C";
            }
            else //Win
            {
                if (oppScore == 1) return "B";
                else if (oppScore == 2) return "C";
                else if (oppScore == 3) return "A";
            }
            return "";
        }

        private int ScoreRound(string o, string m)
        {
            //Its a draw
            if (GetShapeScore(o) == GetShapeScore(m))
            {
                return 3 + GetShapeScore(m); //Tied
            }else if (DidYouWin(o, m))
            {
                return 6 + GetShapeScore(m); //Won
            }
            return GetShapeScore(m); //Lost
        }

        private bool DidYouWin(string o, string m)
        {
            var opp = GetShapeScore(o);
            var mine = GetShapeScore(m);
            if (opp == 1 && mine == 2) return true;
            if (opp == 1 && mine == 3) return false;
            if (opp == 2 && mine == 1) return false;
            if (opp == 2 && mine == 3) return true;
            if (opp == 3 && mine == 1) return true;
            if (opp == 3 && mine == 2) return false;
            return false;
        }

        private int GetShapeScore(string s)
        {
            switch (s)
            {
                //Rock
                case "A":
                case "X":
                    return 1;
                //Paper
                case "B":
                case "Y":
                    return 2;
                //Scissors
                case "C":
                case "Z":
                    return 3;
            }
            return 0;
        }
    }
}
